package com.vulcannis.icfp2002.bots;

import java.util.*;

import com.vulcannis.icfp2002.*;
import com.vulcannis.icfp2002.game.*;
import com.vulcannis.icfp2002.util.*;
import com.vulcannis.icfp2002.game.Package;

public class DeliveryBot implements Bot
{
	public DeliveryBot( final PackagePrioritizer packagePrioritizer, final PathPlotter plotter, final Bidder bidder )
	{
		this.packagePrioritizer = packagePrioritizer;
		this.plotter = plotter;
		this.bidder = bidder;
		unreachablePoints = new HashSet( );
		lastFourActions = new ClientAction[ 4 ];
	}
	
	public void startTurn( final GameState state )
	{
		final Player us = state.getOurPlayer( );
		final Point loc = us.getLocation( );
		final long money = state.getMoney( );
		action = DEFAULT_ACTION;
		
		if( path != null && path.haveArrived( ) ) {
			System.out.println( "Arrived at destination." );
			path = null;
		} else
			System.out.println( "Current path: " + path );
		
		final Set deliveryLocations = us.getDeliveryLocations( );
		// If we can drop packages, do so.
		if( deliveryLocations.contains( loc ) ) {
			final Set deliverables = us.getDeliverablePackages( );
			System.out.println( "Dropping packages: " + deliverables );
			action = bidder.getDropBid( state , deliverables );
		} else if( !unreachablePoints.isEmpty( ) && !deliveryLocations.isEmpty( ) && SetUtil.subtraction( deliveryLocations, unreachablePoints ).isEmpty( ) ) {
			System.out.println( "Can't deliver any of our packages, jettisoning them." );
			state.getPackageInfo( ).addToIgnoreList( Collections.singleton( loc ) );
			action = bidder.getDropBid( state, us.getPackages( ) );
		} else {
			final Set droppedPackagesPresent = state.getPackageInfo( ).getPackages( loc );
			final Set droppedPackagesPickupable = packagePrioritizer.getPickupSet( state, droppedPackagesPresent );
			
			// If there are packages to pick up, do so.
			if( !droppedPackagesPickupable.isEmpty( ) ) {
				System.out.println( "Picking up packages: " + droppedPackagesPickupable );
				action = bidder.getPickupBid( state, droppedPackagesPickupable );
				path = null;
			} else if( path == null ) {
				System.out.println( "No path set, seeking..." );
				path = plotter.getNextPath( state, unreachablePoints );
				
				if( path != null ) {
					action = bidder.getMoveBid( state, path.getNext( ) );
				}
			} else {
				action = bidder.getMoveBid( state, path.getNext( ) );
			}
		}
	}
	
	public ClientAction getAction( )
	{
		lastFourActions[ 0 ] = lastFourActions[ 1 ];
		lastFourActions[ 1 ] = lastFourActions[ 2 ];
		lastFourActions[ 2 ] = lastFourActions[ 3 ];
		lastFourActions[ 3 ] = action;
		return action;
	}
	
	public void endTurn( final GameState state, final boolean actionSuccessful )
	{
		if( isStuck( state ) && path != null ) {
			final Point destination = path.getDestination( );
			System.out.println( "We're stuck, adding " + destination + " to unreachables." );
			unreachablePoints.add( destination );
			path = null;
		} else if( !actionSuccessful )
			path = null;
	}
	
	public static void main( String args[ ] )
	{
		Main.runGame( args, new DeliveryBot( new LargestPackagesFirst( ), new DeliveryPlotter( ), new ConstantBidder( ) ) );
	}
	
	protected boolean isStuck( final GameState state )
	{
		if( state.getOurPlayer( ).isStuck( ) && action instanceof MoveAction )
			return true;
		if( lastFourActions[ 0 ] != null )
			return lastFourActions[ 0 ].equals( lastFourActions[ 2 ] ) && lastFourActions[ 1 ].equals( lastFourActions[ 3 ] ) &&
					!lastFourActions[ 0 ].equals( lastFourActions[ 1 ] ) && !lastFourActions[ 2 ].equals( lastFourActions[ 3 ] );
		return false;
	}
	
	protected ClientAction action = null;
	protected Set unreachablePoints;
	protected ClientAction lastFourActions[ ];

	private Path path = null;
	private PackagePrioritizer packagePrioritizer;
	private PathPlotter plotter;
	private Bidder bidder;
	
	private static final ClientAction DEFAULT_ACTION = new PickupAction( 1, Collections.EMPTY_SET );
}
