package com.vulcannis.icfp2002.bots;

import java.util.*;

import com.vulcannis.icfp2002.game.*;
import com.vulcannis.icfp2002.util.*;

public class DeliveryPlotter implements PathPlotter
{
	public Path getNextPath( final GameState state, final Set unreachables )
	{
		Set destinations = null;
		int tryCount = 0;
		do {
			destinations = getDestinations( state, unreachables );
		} while( destinations.isEmpty( ) && ++tryCount < 2 );
		
		if( destinations.isEmpty( ) )
			state.getPackageInfo( ).clearIgnoreList( );

		return computePath( destinations, state );
	}
	
	protected Set getDestinations( final GameState state, final Set unreachables )
	{
		Set destinations = SetUtil.subtraction( state.getOurPlayer( ).getDeliveryLocations( ), unreachables );
		if( destinations.isEmpty( ) ) {
			System.out.println( "No delivery locations..." );
			destinations = SetUtil.subtraction( state.getPackageInfo( ).getKnownPackageLocations( ), unreachables );
		}
		if( destinations.isEmpty( ) ) {
			System.out.println( "No known packages..." );
			destinations = SetUtil.subtraction( state.getPackageInfo( ).getPossiblePackageLocations( ), unreachables );
		}
		if( destinations.isEmpty( ) ) {
			System.out.println( "No possible packages...removing bases from ignore list." );
//			state.getPackageInfo( ).clearIgnoreList( );
			state.getPackageInfo( ).addToPossibleLocations( state.getMap( ).getBases( ) );
		}
/*		destinations = SetUtil.subtraction( state.getPackageInfo( ).getKnownPackageLocations( ), unreachables );
		}
		if( destinations.isEmpty( ) ) {
			System.out.println( "No known packages..." );
			destinations = SetUtil.subtraction( state.getPackageInfo( ).getPossiblePackageLocations( ), unreachables );
		}
		if( destinations.isEmpty( ) ) {
			System.out.println( "No possible packages..." );
			destinations = SetUtil.subtraction( state.getMap( ).getBases( ), unreachables );
		}
		if( destinations.isEmpty( ) ) {
			unreachables.clear( );
		}
*/		return destinations;
	}
	
	protected Path computePath( final Set destinations, final GameState state )
	{
		if( !destinations.isEmpty( ) ) {
			final Point packageLocations[ ] = (Point[ ])destinations.toArray( new Point[ destinations.size( ) ] );
			try {
				final Point points[ ] = state.getMap( ).getShortestPath( state.getOurPlayer( ).getLocation( ), new AnyDestination( packageLocations ) );
				if( points == null ) {
					System.out.println( "Could not find a path to: " + destinations );
					state.getPackageInfo( ).addUnreachablePoints( destinations );
					return null;
				}
				Path route = new Path( points );
				if( !route.haveArrived( ) ) {
					System.out.println( "Path found: " + route );
					return route;
				}
			}
			catch( InvalidDirectionException e ) {
				// GAH!
			}
		}
		return null;
	}
}
