package com.vulcannis.icfp2002.bots;

import java.util.Set;

import com.vulcannis.icfp2002.game.*;
import com.vulcannis.icfp2002.util.*;

public class ExponentialFailureBidder implements Bidder {

	public ExponentialFailureBidder( )
	{
		this( 2, 0.5, new AlwaysMoveFirst( false ) );
	}
	
	public ExponentialFailureBidder( final double rate, final double ratio, final MovePrioritizer movePrioritizer )
	{
		failureCount = 0;
		this.rate = rate;
		this.ratio = ratio;
		this.movePrioritizer = movePrioritizer;
	}
	
	public void update( final GameState state, final boolean actionSuccessful )
	{
		if( !actionSuccessful && !movePrioritizer.isConflict( ) ) {
			failureCount++;
			System.out.println( "Incrementing failure count, now at " + failureCount );
		}
	}

	public MoveAction getMoveBid( final GameState state, final Direction dir )
	{
		final Point dest = dir.move( state.getOurPlayer( ).getLocation( ) );
		final long bid = getBid( state, dest );
		if( movePrioritizer.shouldMoveFirst( state, dir ) )
			return new MoveAction( bid, dir, state );
		else
			return new MoveAction( -bid, dir, state );
	}

	public PickupAction getPickupBid( final GameState state, final Set packages )
	{
		final Point dest = state.getOurPlayer( ).getLocation( );
		return new PickupAction( getBid( state, dest ), packages );
	}

	public DropAction getDropBid( final GameState state, final Set packages )
	{
		final Point dest = state.getOurPlayer( ).getLocation( );
		return new DropAction( getBid( state, dest ), packages );
	}
	
	private long getBid( final GameState state, final Point dest )
	{
		if( !state.isLocationContestable( dest ) && !state.isLocationContestable( state.getOurPlayer( ).getLocation( ) ) ) {
//			System.out.println( "Destination not contestable, resetting failure count." );
			return 1;
//			failureCount = 0;
		}
		System.out.println( "Destination contestable, bidding." );
		final long wantToPay = (long)Math.pow( rate, failureCount );
		final long maxToPay = Math.max( (long)( state.getMoney( ) * ratio ), 1 );
		return Math.min( wantToPay, maxToPay );
	}
	
	private int failureCount;
	private double rate;
	private double ratio;
	private MovePrioritizer movePrioritizer;
}
