package com.vulcannis.icfp2002.game;

import com.vulcannis.icfp2002.util.*;

public final class Direction
{
	public final static Direction NORTH = new Direction( "N", 0, 1 );
	public final static Direction SOUTH = new Direction( "S", 0, -1 );
	public final static Direction WEST = new Direction( "W", -1, 0 );
	public final static Direction EAST = new Direction( "E", 1, 0 );
	
	public static Direction getDirection( final char dir ) throws InvalidDirectionException
	{
		switch( dir ) {
			case 'N': {
				return NORTH;
			}
			
			case 'S': {
				return SOUTH;
			}
			
			case 'E': {
				return EAST;
			}
			
			case 'W': {
				return WEST;
			}
		}
		throw new InvalidDirectionException( "Unknown direction \"" + dir + "\"" );
	}
	
	public static Direction delta( Point from, Point to ) throws InvalidDirectionException
	{
		final int deltaX = from.getX( ) - to.getX( );
		final int deltaY = from.getY( ) - to.getY( );
		if( deltaX == 1 && deltaY == 0 )
			return WEST;
		if( deltaX == -1 && deltaY == 0 )
			return EAST;
		if( deltaX == 0 && deltaY == 1 )
			return SOUTH;
		if( deltaX == 0 && deltaY == -1 )
			return NORTH;
		throw new InvalidDirectionException( "" + from + " and " + to + " are not adjacent." );
	}
	
	public Point move(Point p) {
		return new Point(p.getX() + deltaX, p.getY() + deltaY);
	}
	
	public String toString( )
	{
		return text;
	}
	
	private Direction( String text, int dx, int dy )
	{
		this.text = text;
		deltaX = dx;
		deltaY = dy;
	}
	
	private String text;
	private int deltaX;
	private int deltaY;
}
