package com.vulcannis.icfp2002.game;

import com.vulcannis.icfp2002.util.*;
import java.util.*;
import java.io.*;

public final class GameMap implements AStarMap {
	/**
	 * Creates a map given the width, height, and a two-dimensional
	 * array of characters encoded per contest rules.
	 */
	public GameMap( BufferedReader in ) throws InvalidStateInformationException, IOException
	{
		final String temp = in.readLine( );
System.out.println( temp );
		final String[] splitLine = temp.split(" ");
		width = Integer.parseInt(splitLine[0]);
		height =  Integer.parseInt(splitLine[1]);
		map = new byte[ width ][ height ];
		bases = new HashSet( );
		for( int y = 0; y < height; y++ ) {
			final String line = in.readLine();
System.out.println( line );
			for( int x = 0; x < width; x++ ) {
				switch( line.charAt( x ) ) {
					case PLAIN_CHAR: {
						map[ x ][ y ] = PLAIN;
						break;
					}

					case WATER_CHAR: {
						map[ x ][ y ] = WATER;
						break;
					}

					case WALL_CHAR: {
						map[ x ][ y ] = WALL;
						break;
					}

					case BASE_CHAR: {
						map[ x ][ y ] = BASE;
						bases.add( new Point( x + 1, y + 1 ) );
						break;
					}

					default: {
						throw new InvalidStateInformationException( "Unknown map character \"" + splitLine[x].charAt(0) + "\"" );
					}
				}
			}
		}
		astar = new AStar( this );
	}
	
	public int getHeight( )
	{
		return height;
	}

	public int getWidth( )
	{
		return width;
	}

	public boolean isPassable( int x, int y )
	{
		if( x <= 0 || x > width || y <= 0 || y > height )
			return false;
		final byte type = map[ x - 1 ][ y - 1 ];
		return type == PLAIN || type == BASE;
	}
	
	public Point[ ] getShortestPath( final Point start, final Destination dest )
	{
		return astar.getShortestPath( start, dest );
	}
	
	public Set getBases( )
	{
		return bases;
	}
	
	public String toString( )
	{
		StringBuffer buffer = new StringBuffer( );
		for( int y = height - 1; y >= 0; y-- ) {
			for( int x = 0; x < width; x++ ) {
				buffer.append( CHARS[ map[ x ][ y ] ] );
			}
			buffer.append( '\n' );
		}
		return buffer.toString( );
	}
	
	private final int width, height;
	private final byte map[ ][ ];
	private Set bases;
	private AStar astar;
	
	public static final byte PLAIN = 0;
	public static final byte WATER = 1;
	public static final byte WALL = 2;
	public static final byte BASE = 3;

	private final static char PLAIN_CHAR = '.';
	private final static char WATER_CHAR = '~';
	private final static char WALL_CHAR = '#';
	private final static char BASE_CHAR = '@';
	private final static char CHARS[ ] = { PLAIN_CHAR, WATER_CHAR, WALL_CHAR, BASE_CHAR };
}
