package com.vulcannis.icfp2002.game;

import java.util.*;

import com.vulcannis.icfp2002.util.*;

public final class GameState {
	public GameState( final GameMap map )
	{
		this.map = map;
		packageManager = new PackageManager( );
		playerManager = new PlayerManager( );
		ourPlayer = null;
		money = 0;
		previousLocation = null;
	}
	
	public void update( final PackageUpdate update )
	{
		packageManager.update( ourPlayer.getLocation( ), update );
	}
	
	public void update( final ClientConfig config )
	{
		this.config = config;
		money = getStartingMoney( );
	}
	
	public void update( final PlayerUpdate update )
	{
		if( ourPlayer != null )
			previousLocation = ourPlayer.getLocation( );
		playerManager.update( update );
		packageManager.update( update );
		if( ourPlayer == null )
			ourPlayer = playerManager.getPlayer( config.getUid( ) );
	}
	
	public void update( final ClientAction action )
	{
		money -= Math.abs( action.getBid( ) );
	}
	
	public GameMap getMap( )
	{
		return map;
	}
	
	public Player getOurPlayer( )
	{
		return ourPlayer;
	}
	
	public Set getOpponents( )
	{
		return SetUtil.subtraction( playerManager.getPlayers( ), Collections.singleton( ourPlayer ) );
	}
	
	public PackageManager getPackageInfo( )
	{
		return packageManager;
	}
	
	public int getMaxWeight( )
	{
		return config.getCapacity( );
	}
	
	public long getStartingMoney( )
	{
		return config.getMoney( );
	}
	
	public long getMoney( )
	{
		return money;
	}
	
	public boolean isEnemyAdjacentTo( final Point loc )
	{
		final Player us = getOurPlayer( );
		final Iterator it = playerManager.getPlayers( ).iterator( );
		while( it.hasNext( ) ) {
			final Player player = (Player)it.next( );
			if( !player.equals( us ) ) {
				if( loc.isAdjacentTo( player.getLocation( ) ) )
					return true;
			}
		}
		return false;
	}
	
	public boolean isLocationContestable( final Point loc )
	{
		final Player us = getOurPlayer( );
		final Iterator it = playerManager.getPlayers( ).iterator( );
		while( it.hasNext( ) ) {
			final Player player = (Player)it.next( );
			if( !player.equals( us ) ) {
				if( loc.equals( player.getLocation( ) ) || loc.isAdjacentTo( player.getLocation( ) ) )
					return true;
			}
		}
		return false;
	}
	
	public Point getPreviousLocation( )
	{
		return previousLocation;
	}
	
	private GameMap map;
	private PackageManager packageManager;
	private PlayerManager playerManager;

	private ClientConfig config;	
	private Player ourPlayer;
	private long money;
	private Point previousLocation;
}
