package com.vulcannis.icfp2002.game;

import java.util.*;

import com.vulcannis.icfp2002.util.*;

public final class Player implements PackageCarrier
{
	public Player(int i, Point p) {
		id = i;
		location = p;
		packages = new HashSet( );
		deliveryLocations = null;
		weight = 0;
	}
	
	public int getUid() {
		return id;
	}
	
	public Point getLocation() {
		return location;
	}
	
	public void setLocation(Point p) {
		location = p;
	}
	
	public Set getDeliveryLocations( )
	{
		if( deliveryLocations == null ) {
			if( packages.isEmpty( ) )
				deliveryLocations = Collections.EMPTY_SET;
			else {
				deliveryLocations = new HashSet( );
				final Iterator it = packages.iterator( );
				while( it.hasNext( ) ) {
					final Package element = (Package)it.next( );
					deliveryLocations.add( element.getDestination( ) );
				}
			}
		}
		return deliveryLocations;
	}
	
	public void addPackage( final Package pack )
	{
		packages.add( pack );
		deliveryLocations = null;
		weight += pack.getWeight( );
	}
	
	public void removePackage( final Package pack )
	{
		packages.remove( pack );
		deliveryLocations = null;
		weight -= pack.getWeight( );
	}
	
	/**
	 * @return all packages currently being carried.
	 */
	public Set getPackages( )
	{
		return packages;
	}
	
	/**
	 * @return the total weight of all packages currently being carried.
	 */
	public int getCurrentWeight( )
	{
		return weight;
	}
	
	/**
	 * @return the set of packages that if dropped at the current location will be delivered.
	 */
	public Set getDeliverablePackages( )
	{
		final Set deliverables = new HashSet( );
		final Iterator it = packages.iterator( );
		while( it.hasNext( ) ) {
			final Package element = (Package)it.next( );
			if( location.equals( element.getDestination( ) ) )
				deliverables.add( element );
		}
		return deliverables;
	}
	
	public boolean isStuck( )
	{
		return isStuck;
	}
	
	public void setStuck( final boolean value )
	{
		isStuck = value;
	}
	
	public int hashCode( )
	{
		return id;
	}
	
	public boolean equals( Object other )
	{
		if( other != null && other instanceof Player ) {
			final Player p = (Player)other;
			return p.id == id;
		}
		return false;
	}
	
	public String toString( )
	{
		final StringBuffer buffer = new StringBuffer( );
		buffer.append( "ID: " );
		buffer.append( id );
		buffer.append( " @" );
		buffer.append( location );
		buffer.append( " Carrying: " );
		buffer.append( packages );
		buffer.append( " Current Weight: " );
		buffer.append( weight );
		return buffer.toString( );
	}
	
	private int id;
	private Point location;
	private Set packages;
	private Set deliveryLocations;
	private int weight;
	private boolean isStuck;
}
