package com.vulcannis.icfp2002.game;

import com.vulcannis.icfp2002.util.*;
import java.util.*;

public final class PlayerManager
{
	public PlayerManager( )
	{
		players = new HashSet( );
		idMap = new HashMap( );
		locationMap = new HashMap( );
	}

	public void update( PlayerUpdate update )
	{
		PlayerAction[] actions = update.getPlayerActions( );
		for (int i = 0; i < actions.length; i++) {
			if (getPlayer(actions[i].getUid()) == null)
				// Add a new player - we are assuming the first action we get
				// is the (X, Y) location of the new robot.
				addPlayer(new Player(actions[i].getUid(), ((PlayerAbsoluteMove)actions[i]).getPoint()));
			else {
				// Existing player.
				Player currentPlayer = getPlayer(actions[i].getUid());
				removePlayer(actions[i].getUid());
				actions[i].updatePlayer(currentPlayer);
				addPlayer(currentPlayer);
			}
		}
	}

	public Player getPlayer( int id )
	{
		return (Player)idMap.get(new Integer(id));
	}
	
	public Set getPlayersAtLocation( Point p )
	{
		return (Set)locationMap.get(p);
	}
	
	public Set getPlayers( )
	{
		return players;
	}
	
	private void addPlayer(Player p) {
		players.add( p );
		idMap.put(new Integer(p.getUid()), p);
		if (locationMap.get(p.getLocation()) == null)
			locationMap.put(p.getLocation(), new HashSet());
		((Set)locationMap.get(p.getLocation())).add(p);
	}
	
	private void removePlayer(int id) {
		Player p = (Player)idMap.remove(new Integer(id));
		players.remove( p );
		Set set = (Set)locationMap.get(p.getLocation());
		set.remove(p);
		if (set.isEmpty())
			locationMap.remove(p.getLocation());
	}
	
	private Set players;
	private Map idMap;
	private Map locationMap;
}
