package com.vulcannis.icfp2002.game;

import java.io.*;
import java.util.*;

import com.vulcannis.icfp2002.net.*;

public final class PlayerUpdate {
	public PlayerUpdate(BufferedReader in) throws IOException, InvalidStateInformationException, GameOverException
	{
		final ArrayList playerActionList =  new ArrayList( );
		final ArrayList packageActionList = new ArrayList( );

		final String text = in.readLine( );
		if( text == null || text.indexOf( "died" ) != -1 )
			throw new GameOverException( );
System.out.println( "Player update: \"" + text + "\"" );
		final String playerTurns[ ] = text.split( "#" );
		for (int t = 1; t < playerTurns.length; t++) {
			final String[] playerActions = playerTurns[t].split(" ");
			// If a no-op
			final int player = Integer.parseInt(playerActions[0]);
			if (playerActions.length == 1)
				playerActionList.add(new PlayerNoOp(player));
			else {
				for (int a = 1; a < playerActions.length; a++) {
					switch (playerActions[a].charAt(0)) {
						case 'N':
							playerActionList.add(new PlayerMove(player, Direction.NORTH));
						 	break;
						case 'S':
							playerActionList.add(new PlayerMove(player, Direction.SOUTH));
							break;
						case 'W':
							playerActionList.add(new PlayerMove(player, Direction.WEST));
							break;
						case 'E':
							playerActionList.add(new PlayerMove(player, Direction.EAST));
							break;
						case 'P': {
							final PlayerPickup action = new PlayerPickup(player, Integer.parseInt(playerActions[++a]));
							playerActionList.add( action );
							packageActionList.add( action );
							break;
						}
						case 'D': {
							final PlayerDrop action = new PlayerDrop(player, Integer.parseInt(playerActions[++a]));
							playerActionList.add( action );
							packageActionList.add( action );
							break;
						}
						case 'X':
							playerActionList.add(new PlayerAbsoluteMove(player, Integer.parseInt(playerActions[a+1]), Integer.parseInt(playerActions[a+3])));
							a += 3;
							break;
						default:
							throw new InvalidStateInformationException("Invalid state character \"" + playerActions[a].charAt(0) + "\"");
					}
				}
			}
		}
		
		playerActions = (PlayerAction[ ])playerActionList.toArray( new PlayerAction[ playerActionList.size( ) ] );
		packageActions = (PackageAction[ ])packageActionList.toArray( new PackageAction[ packageActionList.size( ) ] );
	}

	public PlayerAction[ ] getPlayerActions( )
	{
		return playerActions;
	}
	
	public PackageAction[ ] getPackageActions( )
	{
		return packageActions;
	}

	public PlayerAction playerActions[ ];
	public PackageAction packageActions[ ];
}
