package com.vulcannis.icfp2002.net;

import java.net.*;
import java.io.*;

import com.vulcannis.icfp2002.*;
import com.vulcannis.icfp2002.game.*;

public final class NetworkServer implements GameServer
{
	/** @param h The remote hostname
	 *  @param p The remort port
	 */
	public NetworkServer(String h, int p) throws GameSocketException
	{
		try {
			socket = new Socket(h, p);
		}
		catch (IOException e) {
			throw new GameSocketException("Can't open connection to server.", e);
		}
		catch (SecurityException e) {
			throw new GameSocketException("Can't open connection to server.", e);
		}
	}
	
	public void addPlayer(GameClient client) throws GameException
	{
		try {
			final BufferedReader sin = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			final PrintWriter sout = new PrintWriter( socket.getOutputStream(), true );
			
			System.out.println( "Sending login request..." );
			
			sout.println("Player");
			
			System.out.println( "Receiving game map..." );
			final GameMap gameMap = new GameMap(sin);
			System.out.println( "Receiving client configuration..." );
			final ClientConfig clientConfig = new ClientConfig(sin);
			System.out.println( "Receiving initial update..." );
			final PlayerUpdate initalUpdate = new PlayerUpdate(sin);

			client.initPlayer(gameMap, clientConfig, initalUpdate);
			
			boolean done = false;
			while( !done ) {
				try { 
					final PackageUpdate packageUpdate = new PackageUpdate( sin );
					final ClientAction action = client.startTurn( packageUpdate );
					sout.println( action );
					final PlayerUpdate playerUpdate = new PlayerUpdate( sin );
					client.endTurn( playerUpdate );
				}
				catch( GameOverException e ) {
					done = true;
				}
			}
			
			System.out.println( "Finished game." );
			
			String text = sin.readLine( );
			while( text != null ) {
				System.out.println( text );
				text = sin.readLine( );
			}
		}
		catch( IOException e ) {
			throw new GameSocketException( e );
		}
	}

	private Socket socket;
}
