package com.vulcannis.icfp2002.util;

import java.io.*;
import java.util.Random;
import com.vulcannis.icfp2002.game.*;

public class TestAStar {

	public static void main( String args[ ] )
	{
		try {
			if( args.length == 0 ) {
				Destination dest = new SingleDestination( 1000, 1000 );
				{
					System.out.println( "ClearMap:" );
					AStar algo = new AStar( new ClearMap( 1000, 1000 ) );
					for( int loop = 0; loop < 5; loop++ ) {
						long start = System.currentTimeMillis( );
						Path path = new Path( algo.getShortestPath( new Point( 1, 1 ), dest ) );
						System.out.println( System.currentTimeMillis( ) - start );
						System.out.println( path );
					}
				}
				{
					System.out.println( "OutlineMap:" );
					AStar algo = new AStar( new OutlineMap( 1000, 1000 ) );
					for( int loop = 0; loop < 5; loop++ ) {
						long start = System.currentTimeMillis( );
						Path path = new Path( algo.getShortestPath( new Point( 1, 1 ), dest ) );
						System.out.println( System.currentTimeMillis( ) - start );
						System.out.println( path );
					}
				}
				{
					System.out.println( "RandomMap:" );
					AStar algo = new AStar( new RandomMap( 1000, 1000 ) );
					for( int loop = 0; loop < 5; loop++ ) {
						long start = System.currentTimeMillis( );
						Path path = new Path( algo.getShortestPath( new Point( 1, 1 ), dest ) );
						System.out.println( System.currentTimeMillis( ) - start );
						System.out.println( path );
					}
				}
				{
					System.out.println( "ArrowMap:" );
					AStar algo = new AStar( new ArrowMap( 1000, 1000 ) );
					for( int loop = 0; loop < 5; loop++ ) {
						long start = System.currentTimeMillis( );
						Path path = new Path( algo.getShortestPath( new Point( 1, 1 ), dest ) );
						System.out.println( System.currentTimeMillis( ) - start );
						System.out.println( path );
					}
				}
			} else {
				String mapName = args[ 0 ];
				int width, height;
				BufferedReader reader = new BufferedReader( new FileReader( mapName ) );
				/*width = Integer.parseInt( reader.readLine( ) );
				height = Integer.parseInt( reader.readLine( ) );
				char data[ ][ ] = new char[ width ][ height ];
				for( int y = 0; y < height; y++ ) {
					String line = reader.readLine( );
					for( int x = 0; x < width; x++ )
						data[ x ][ y ] = line.charAt( x );
				}
				*/
				AStarMap map = new GameMap( reader );

				int startX = 1, startY = 1;
				if( args.length > 1 ) {
					startX = Integer.parseInt( args[ 1 ] );
					startY = Integer.parseInt( args[ 2 ] );
				}
				int destX = map.getWidth( ), destY = map.getHeight( );
				if( args.length > 3 ) {
					destX = Integer.parseInt( args[ 3 ] );
					destY = Integer.parseInt( args[ 4 ] );
				}
				
				System.out.println( map );
				AStar algo = new AStar( map );
				long start = System.currentTimeMillis( );
				Path path = new Path( algo.getShortestPath( new Point( startX, startY ), new SingleDestination( destX, destY ) ) );
				System.out.println( System.currentTimeMillis( ) - start );
				algo.dumpClosed( );
				System.out.println( path );
			}
		}
		catch( Exception e ) {
			e.printStackTrace( );
		}
	}
}

class ClearMap implements AStarMap
{
	public ClearMap( int width, int height )
	{
		this.width = width;
		this.height = height;
	}
	
	public int getHeight( )
	{
		return height;
	}

	public int getWidth( )
	{
		return height;
	}

	public boolean isPassable( int x, int y )
	{
		return x > 0 && x <= width && y > 0 && y <= height;
	}

	private int width, height;
}

class OutlineMap implements AStarMap
{
	public OutlineMap( int width, int height )
	{
		this.width = width;
		this.height = height;
	}
	
	public int getHeight( )
	{
		return height;
	}

	public int getWidth( )
	{
		return height;
	}

	public boolean isPassable( int x, int y )
	{
		return ( ( x == 1 || x == width ) && y > 0 && y <= height ) ||
		        ( ( y == 1 || y == height ) && x > 0 && x <= width );
	}

	private int width, height;
}

class RandomMap implements AStarMap
{
	public RandomMap( int width, int height )
	{
		this.width = width;
		this.height = height;
		obstructed = new boolean[ width ][ height ];
		Random rng = new Random( );
		for( int loop = 0; loop < Math.min( width, height ); loop++ )
			obstructed[ rng.nextInt( width ) ][ rng.nextInt( height ) ] = true;
		obstructed[ 0 ][ 0 ] = false;
	}
	
	public int getHeight( )
	{
		return height;
	}

	public int getWidth( )
	{
		return height;
	}

	public boolean isPassable( int x, int y )
	{
		return x > 0 && x <= width && y > 0 && y <= height && !obstructed[ x - 1 ][ y - 1 ];
	}

	private int width, height;
	private boolean obstructed[ ][ ];
}

class ArrowMap implements AStarMap
{
	public ArrowMap( int width, int height )
	{
		this.width = width;
		this.height = height;
	}
	
	public int getHeight( )
	{
		return height;
	}

	public int getWidth( )
	{
		return height;
	}

	public boolean isPassable( int x, int y )
	{
		return x > 0 && x <= width && y > 0 && y <= height && ( Math.abs( x - y ) < 2 );
	}

	private int width, height;
}
